@@ -1,3 +1,5 @@
+globalVariables('ind')
+
 setGeneric('densityplot')
 
 setMethod('densityplot',
@@ -6,9 +8,10 @@ setMethod('densityplot',
             xlab='', ylab='', main='', col='black',...){
             dat <- raster2dat(x, maxpixels=maxpixels)
             densityplot(dat,
-                        data=NULL,
-                        pch='.', col=col,
-                        xlab=xlab, ylab=ylab, ...) 
+                        data = NULL,
+                        pch = '.', col = col,
+                        xlab = xlab, ylab = ylab, main = main,
+                        ...) 
           }
           )
 
@@ -17,32 +20,39 @@ setMethod('densityplot',
           signature(x='RasterStackBrick', data='missing'),
           definition=function (x, data=NULL, layers, FUN,
             maxpixels = 1e+05,
-            xlab='', ylab='', main='',
+            xlab = '', ylab = '', main = '',
             par.settings=rasterTheme(),...){
             if (!missing(layers)) x <- subset(x, layers)
             nl=nlayers(x)
             if (nl > 1) {
               dat <- raster2dat(x, FUN, maxpixels)
               p <- densityplot(~values,
-                               data=dat, groups=ind,
+                               data = dat, groups = ind,
                                ## scales=list(x=list(relation='free'),
                                ##   y=list(relation='free', draw=FALSE)),
-                               breaks=100, par.settings=par.settings, pch='.',
-                               xlab=xlab, ylab=ylab,
-                               panel=panel.superpose,
-                               panel.groups=function(x, group.value, col.line,...){
-                                 panel.densityplot(x, col.line=col.line, plot.points=FALSE,...)
-                                 d <- density(x, na.rm=1)
+                               breaks = 100,
+                               par.settings = par.settings, pch = '.',
+                               xlab = xlab, ylab = ylab, main = main,
+                               panel = panel.superpose,
+                               panel.groups = function(x, group.value, col.line,...){
+                                   panel.densityplot(x,
+                                                     col.line = col.line,
+                                                     plot.points = FALSE,
+                                                     ...)
+                                 d <- density(x, na.rm = TRUE)
                                  i <- which.max(d$y)
                                  ltext(d$x[i],d$y[i],
                                        group.value,
-                                       adj=c(0.3,0),
-                                       col=col.line,
-                                       cex=0.7)
+                                       adj = c(0.3,0),
+                                       col = col.line,
+                                       cex = 0.7)
                                }
                                )
             } else {
-              p <- densityplot(x, maxpixels = maxpixels, main = main, xlab=xlab, ylab=ylab,...)
+                p <- densityplot(x,
+                                 maxpixels = maxpixels,
+                                 main = main, xlab=xlab, ylab=ylab,
+                                 ...)
             }
             p
           }
