@@ -3,7 +3,7 @@ setGeneric('levelplot')
 setMethod('levelplot',
           signature(x = 'Raster', data = 'missing'),
           definition = function(x, data = NULL, layers,
-              margin=!(any(is.factor(x))), FUN.margin=mean,
+              margin = list(), 
               maxpixels = 1e5,
               par.settings = rasterTheme(),
               between = list(x = 0.5, y = 0.2),
@@ -13,24 +13,31 @@ setMethod('levelplot',
               main = NULL,
               names.attr,
               scales = list(),
-              scales.margin = NULL, axis.margin = FALSE,
               xscale.components = xscale.raster,
               yscale.components = yscale.raster,
               zscaleLog = NULL,
               colorkey = list(space = 'right'),
               panel = panel.levelplot, pretty = FALSE, 
               contour = FALSE, region = TRUE, labels = FALSE,
+              FUN.margin = NULL,
+              scales.margin = NULL, axis.margin = NULL,
               ..., att = 1L) {
-
+              
+              ## Extract components from par.settings              
+              if (is.function(par.settings))
+                  par.settings <- par.settings()
+              ## Subset the object if layers are defined
               if (!missing(layers)) {
                   object <- subset(x, subset=layers)
-              } else {object <- x}
-
+              } else {
+                  object <- x
+              }
+              
               ## The plot display a sample of the whole object defined
               ## with maxpixels
               objectSample <- sampleRegular(object, size=maxpixels,
                                             asRaster=TRUE)
-
+
               ## Is factor?
               factorLayers <- is.factor(object)
               isFactor <- all(factorLayers)
@@ -46,25 +53,25 @@ setMethod('levelplot',
                   } else {
                       rat <- as.data.frame(rat[[1]])
                       ratID <- rat$ID
-                      nLevels <- length(ratID)
                       ## choose which level to use for the legend
                       if (is.numeric(att)) att = att + 1
-                      ratLevels <- rat[, att]
+                      ratLevels <- factor(rat[, att])
                   }
                   ## Use factor index (position) instead of code (ratID)
-                  dat <- match(objectSample, ratID)
-                  dat <- as.data.frame(getValues(dat))
+                  idxLevels <- as.numeric(ratLevels)
+                  idxRaster <- subs(objectSample,
+                                    data.frame(ratID, idxLevels))
+                  dat <- as.data.frame(idxRaster)
                   names(dat) <- names(object)
 
                   xy <- xyFromCell(objectSample, 1:ncell(objectSample))
-
                   df <- cbind(xy, dat)
               } else {
                   ## Convert to a data.frame for conventional levelplot
                   df <- as.data.frame(objectSample, xy=TRUE)
                   dat <- df[, -c(1, 2)]
               }
-
+
               ## If zscaleLog is not NULL, transform the values and
               ## choose a function to calculate the labels
               if (identical(FALSE, zscaleLog)) zscaleLog <- NULL
@@ -85,12 +92,13 @@ setMethod('levelplot',
                       zscale.components <- zscale.components.logpower
                   }
                   dat <- log(dat, zlogbase)
+                  ## Update the data content of the original data.frame
+                  df[, -c(1, 2)] <- dat
               }
               ## Calculate the range (for zscale.components)
               zlim <- extendrange(dat,
                                   f=lattice.getOption("axis.padding")$numeric)
-
-
+
               ##aspect and scales(from sp::spplot.grid,
               ##sp::longlat.scales, sp::mapasp)
               bb <- extent(object)
@@ -101,23 +109,29 @@ setMethod('levelplot',
 
                   aspect=(diff(ylim)/diff(xlim))/cos((mean(ylim) * pi)/180)
 
-                  xscale.components <- if (identical(xscale.components, xscale.raster))
-                      xscale.raster.EW
-                  else if (identical(xscale.components, xscale.raster.subticks))
-                      xscale.raster.EWsubticks
-                  else xscale.components
-
-                  yscale.components <- if (identical(yscale.components, yscale.raster))
-                      yscale.raster.NS
-                  else if (identical(yscale.components, yscale.raster.subticks))
-                      yscale.raster.NSsubticks
+                  xscale.components <-
+                      if (identical(xscale.components,
+                                    xscale.raster))
+                          xscale.raster.EW
+                      else if (identical(xscale.components,
+                                         xscale.raster.subticks))
+                          xscale.raster.EWsubticks
+                      else xscale.components
+                  yscale.components <-
+                      if (identical(yscale.components,
+                                    yscale.raster))
+                          yscale.raster.NS
+                      else if (identical(yscale.components,
+                                         yscale.raster.subticks))
+                          yscale.raster.NSsubticks
                   else yscale.components
 
               } else { ## !isLonLat
                   aspect='iso'
               }
-                            
-              if (region==FALSE) colorkey=FALSE
+
+              ## Colorkey 
+              if (region==FALSE) colorkey <- FALSE
 
               colorkey.default <- list(space = 'right')
               if (isTRUE(colorkey)) {
@@ -127,45 +141,33 @@ setMethod('levelplot',
                   has.colorkey <- is.list(colorkey)
               }
 
-              has.margin <- (nlayers(object)==1 && margin)
-              
-              ## Build the zscale.components and colorkey paying attention to zscaleLog
+              ## Build the zscale.components and colorkey paying
+              ## attention to zscaleLog
               if (!is.null(zscaleLog) && !isFactor){
                   zscale <- zscale.components(zlim, zlogbase)
                   colorkey.default = modifyList(colorkey.default,
-                      list(labels = zscale, raster = TRUE, interpolate = TRUE))
+                      list(labels = zscale,
+                           raster = TRUE,
+                           interpolate = TRUE))
               }
-
-              ## Some fixes for the margin
-              if (has.margin){
-                  if (is.function(par.settings)) par.settings <- par.settings()
-                  par.settings = modifyList(par.settings,
-                      list(
-                          layout.widths = list(right.padding = 10),
-                          layout.heights = list(top.padding = 10,
-                              xlab.key.padding = 3)))
-                  
-                  ## put the colorkey at the bottom to leave space for the margin
-                  if (has.colorkey) {
-                      colorkey = modifyList(colorkey, list(space='bottom'))
-                  } else colorkey = FALSE
-              }
-           
+              ## Factor variables need special colorkey
               if (isFactor) {
+                  labs <- levels(ratLevels)
+                  nLabs <- nlevels(ratLevels)
                   ## define the breaks
-                  my.at <- seq(0.5, nLevels + 0.5,
-                               length = nLevels + 1)
+                  my.at <- seq(0.5, nLabs + 0.5,
+                               length = nLabs + 1)
                   ## the labels will be placed vertically centered
-                  my.labs.at <- seq_len(nLevels)
+                  my.labs.at <- seq_len(nLabs)
+
                   colorkey.default <- modifyList(colorkey.default,
                                                  list(
                                                      at = my.at,
-                                                     height = min(1, 0.05*nLevels),
-                                                     labels = list(labels=ratLevels,
+                                                     height = min(1, 0.05*nLabs),
+                                                     labels = list(labels = labs,
                                                          at=my.labs.at))
                                                  )
               }
-
               ## Finally construct colorkey with user definition
               ## (colorkey) and defaults defined by zscale, and
               ## isFactor
@@ -176,24 +178,80 @@ setMethod('levelplot',
                       colorkey = modifyList(colorkey.default, colorkey)
                   }
               }
+
+              ## Construct the margins
+              if (is.logical(margin))
+                  margin <- drawMargin(margin)
 
-              ## Which panel did the user requested? (panel.levelplot or panel.levelplot.raster)
+              if (!is.null(FUN.margin)) {
+                  warning('FUN.margin is deprecated. Use margin as a list instead.')
+                  margin <- modifyList(margin, list(margin = FUN.margin))
+              }
+              if (!is.null(axis.margin)) {
+                  warning('axis.margin is deprecated. Use margin as a list instead.')
+                  margin <- modifyList(margin, list(axis = axis.margin))
+              }
+              if (!is.null(scales.margin)) {
+                  warning('scales.margin is deprecated. Use margin as a list instead.')
+                  margin <- modifyList(margin, list(scales = scales.margin))
+              }
+
+              margin <- do.call('constructMargin', margin)
+              ## Disable margins for multilayer and categorical
+              ## rasters
+              if (nlayers(object) > 1 | any(is.factor(x)))
+                  margin <- drawMargin(FALSE)
+              if (isTRUE(margin$x$draw)){
+                  par.settings <-
+                      modifyList(par.settings,
+                                 list(
+                                     layout.heights = list(
+                                         top.padding = 10,
+                                         xlab.key.padding = 3))
+                                 )
+              }
+              if (isTRUE(margin$y$draw)){
+                  ## put the colorkey at the bottom to leave space for
+                  ## the margin
+                  if (has.colorkey) {
+                      colorkey <- modifyList(colorkey,
+                          list(space='bottom'))
+                  }
+                  par.settings <-
+                      modifyList(par.settings,
+                                 list(
+                                     layout.widths = list(
+                                         right.padding = 10))
+                                 )
+              }
+
+              ## Which panel did the user requested? (panel.levelplot
+              ## or panel.levelplot.raster)
               requestedPanel <- panel
 
               ## Build a custom panel useful for zscaleLog with contour
               panelMixed <- function(...,
                                      contour, region,
-                                     at, labels, levelPanel=requestedPanel,
-                                     lwd=c(1, 0.4) # Two line widths, for main and minor contour lines
+                                     at, labels,
+                                     levelPanel=requestedPanel,
+                                     lwd=c(1, 0.4) # Two line widths,
+                                                   # for main and
+                                                   # minor contour
+                                                   # lines
                                      ){
-                  ##Draw the regions with the panel requested by the user
-                  if (region) levelPanel(..., at=at, contour=FALSE, labels=FALSE)
-                  has.labels <- (is.logical(labels) && labels) || is.list(labels)
+                  ##Draw the regions with the panel requested by the
+                  ##user
+                  if (region) levelPanel(..., at=at,
+                                         contour=FALSE, labels=FALSE)
+                  has.labels <- (is.logical(labels) &&
+                                     labels) ||
+                                         is.list(labels)
                   mainTicks=(as.character(zscale$labels)!=FALSE)
-                  mainLabels <- list(labels=zscale$labels[mainTicks], cex=0.8)
+                  mainLabels <- list(labels=zscale$labels[mainTicks],
+                                     cex=0.8)
                   minorTicks=!mainTicks
-                  ## Contour lines for main divisions (log10Ticks or logpower)
-                  ## Their width is lwd[1]
+                  ## Contour lines for main divisions (log10Ticks or
+                  ## logpower) Their width is lwd[1]
                   panel.contourplot(...,
                                     at=zscale$at[mainTicks], lwd=lwd[1],
                                     labels=if (has.labels) {
@@ -203,18 +261,22 @@ setMethod('levelplot',
                   ## Contour lines for minor divisions (only log10Ticks)
                   if (any(minorTicks)) {
                       panel.contourplot(...,
-                                        at=zscale$at[minorTicks], lwd=lwd[2],
+                                        at=zscale$at[minorTicks],
+                                        lwd=lwd[2],
                                         region=FALSE, contour=TRUE)
                   }
               }
 
-              ## Finally, the panel function depends on zscaleLog and contour
-              panel <- if (!is.null(zscaleLog) && isTRUE(contour) && !isFactor) {
+              ## Finally, the panel function depends on zscaleLog and
+              ## contour
+              panel <- if (!is.null(zscaleLog) &&
+                           isTRUE(contour) &&
+                           !isFactor) {
                   panelMixed
               } else {
                   requestedPanel
               }
-              
+
               ## Names of each panel
               if (missing(names.attr)){
                   names.attr <- names(object)
@@ -224,11 +286,11 @@ setMethod('levelplot',
                       stop('Length of names.attr should match number of layers.')
               }
               ## Build the formula for levelplot
-              form <- as.formula(paste(paste(names(object), collapse='+'), 'x*y', sep='~'))
-
-              ## Update the data content of the original data.frame
-              df[, -c(1, 2)] <- dat
+              form <- as.formula(paste(paste(names(object),
+                                             collapse='+'),
+                                       'x*y', sep='~'))
 
+
               ## For each layer: is the raster completely filled with
               ## NA?  If the object is a multilayer Raster and there
               ## is at least one layer that with non-missing values,
@@ -238,10 +300,10 @@ setMethod('levelplot',
               if (all(is.na(dat))) {
                   region <- FALSE
                   colorkey <- FALSE
-                  margin <- FALSE
+                  margin <- drawMargin(FALSE)
                   pretty <-  TRUE
               }
-
+
               ## And finally, the levelplot call
               p <- levelplot(form, data = df,
                              scales = scales,
@@ -253,32 +315,39 @@ setMethod('levelplot',
                              xscale.components = xscale.components,
                              yscale.components = yscale.components,
                              colorkey = colorkey,
-                             contour = contour, region = region, labels = labels,
+                             contour = contour, region = region,
+                             labels = labels,
                              strip = strip.custom(factor.levels = names.attr),
                              panel = panel, pretty = pretty, ...)
+
+
               ## panel.levelplot uses level.colors to encode values
               ## with colors. It does not work properly with
               ## categorical data and col.regions
               if (isFactor) p <- update(p, at = my.at)
-
+
               ## Plot the margins if required
-              if (nlayers(object)==1 && margin) {
-                  marginsLegend <- list(right=list(
-                                            fun=legendGeneric,
-                                            args=list(p, FUN = FUN.margin,
-                                                scaleAxis = scales.margin$y,
-                                                axis.margin = axis.margin,
-                                                side = 'y')),
-                                        top=list(
-                                            fun=legendGeneric,
-                                            args=list(p, FUN=FUN.margin,
-                                                scaleAxis = scales.margin$x,
-                                                axis.margin = axis.margin,
-                                                side = 'x'))
-                                        )
-                  if (is.null(p$legend)) p$legend <- list()
-                  p$legend <- modifyList(p$legend, marginsLegend)
-
+              if (is.null(p$legend)) p$legend <- list()
+              if (isTRUE(margin$y$draw)) {
+                  right <- list(
+                      fun = legendGeneric,
+                      args = list(p,
+                          FUN = margin$y$FUN,
+                          scaleAxis = margin$y$scales,
+                          axis.margin = margin$y$axis,
+                          side = 'y'))
+                  p$legend <- modifyList(p$legend, list(right = right))
+              }
+              
+              if (isTRUE(margin$x$draw)){
+                  top <- list(
+                      fun = legendGeneric,
+                      args = list(p,
+                          FUN = margin$x$FUN,
+                          scaleAxis = margin$x$scales,
+                          axis.margin = margin$x$axis, 
+                          side = 'x'))
+                  p$legend <- modifyList(p$legend, list(top = top))
               }
               ## Here is the result!
               p
